using System;
using System.Collections.Generic;
using System.Text;

namespace Components.Calendar
{
    /// <summary>
    /// Information about a entry in the agenda
    /// </summary>            
    [Serializable]
    public class MeetingRow : IComparable<MeetingRow>, ICloneable
    {
        #region Variables
        /// <summary>
        /// Internal ID of the meeting
        /// </summary>
        private int vID = -1;
        /// <summary>
        /// Title of the entry
        /// </summary>
        private string vTitle = "";
        /// <summary>
        /// Starting date of the entry
        /// </summary>
        private DateTime vStart = new DateTime();
        /// <summary>
        /// Ending date of the entry
        /// </summary>
        private DateTime vEnd = new DateTime();
        /// <summary>
        /// Data of the entry
        /// </summary>
        private object vEntryData = new object();
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public MeetingRow() { }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the entry data (Information about the entry)
        /// </summary>
        public object MeetingData
        {
            get { return vEntryData; }
            set { vEntryData = value; }
        }
        /// <summary>
        /// Gets or sets the ending date of the entry
        /// </summary>
        public DateTime EndingDate
        {
            get { return vEnd; }
            set { vEnd = value; }
        }
        /// <summary>
        /// Gets or sets the starting date of the entry
        /// </summary>
        public DateTime StartingDate
        {
            get { return vStart; }
            set { vStart = value; }
        }
        /// <summary>
        /// Gets or sets the informations to display in the calendar for the entry
        /// </summary>
        public string Title
        {
            get { return vTitle; }
            set { vTitle = value; }
        }
        /// <summary>
        /// Gets the internal id of the meeting (THIS ID IS FOR INTERNAL USE)
        /// </summary>
        public int ID { get { return this.vID; } }
        #endregion

        #region Methods
        /// <summary>
        /// Compare the contents of a DateToDisplay with this instance
        /// </summary>
        /// <param name="_mr">DateToDisplay to compare</param>
        /// <returns>TRUE if the contents of the two DateToDisplay are equal otherwise FALSE</returns>
        public virtual bool HasSameContents(MeetingRow _mr)
        {
            if (this.EndingDate == _mr.EndingDate
                && this.StartingDate == _mr.StartingDate
                && this.Title == _mr.Title
                && this.MeetingData == _mr.MeetingData
                && this.ID == _mr.ID)
                return true;
            else
                return false;

        }
        /// <summary>
        /// Sets the ID of this DateToDisplay
        /// </summary>
        /// <param name="_ID">_ID of this DateToDisplay</param>  
        /// <exception cref="ArgumentException">If the ID is less than zero</exception>
        public void SetID(int _ID)
        {
            if (_ID < 0)
                throw new ArgumentException("The ID cannot be less than zero!");
            this.vID = _ID;
        }
        /// <summary>
        /// Returns the Meeting in a string which contains all the informations
        /// </summary>
        /// <returns>String which contains all the informations of the meeting</returns>
        public virtual string ToLongString()
        {
            return this.StartingDate.ToShortDateString() + " from " + this.StartingDate.ToShortTimeString()
                + " until " + this.EndingDate.ToShortTimeString()
                + "\r\nTitle: " + this.Title + "\r\nData: " + this.MeetingData.ToString(); ;
        }
        #endregion

        #region Overriden
        /// <summary>
        /// Return the title of the DateToDisplay
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Title;
        }
        #endregion

        #region Operator
        /// <summary>
        /// Returns _DateToDisplay.ToString(). Gets a string which contains the title of _DateToDisplay
        /// </summary>
        /// <param name="_MeetingRow">DateToDisplay to transform into string</param>
        /// <returns>string which is the title of _DateToDisplay</returns>
        public static implicit operator string(MeetingRow _MeetingRow)
        {
            return _MeetingRow.Title;
        }
        #endregion

        #region IComparable<DateToDisplay> Members
        /// <summary>        
        /// Compare this instance of another DateToDisplay on the starting date
        /// </summary>
        /// <param name="other">DateToDisplay to compare at this instance.</param>
        /// <returns> A 32-bit signed integer that indicates the relative order of the objects
        /// being compared. The return value has the following meanings: Value Meaning
        /// Less than zero This object is less than the other parameter.Zero This object
        /// is equal to other. Greater than zero This object is greater than other. </returns>        
        public int CompareTo(MeetingRow other)
        {
            if (other.StartingDate > this.StartingDate)
                return -1;
            else if (other.StartingDate < this.StartingDate)
                return 1;
            else
                return 0;
        }

        #endregion

        #region ICloneable Members
        /// <summary>
        /// A new instance of DateToDisplay with the same data than this instance/// Make a clone copy of this object
        /// </summary>
        /// <returns>A new instance of DateToDisplay with the same data than this instance</returns>
        public object Clone()
        {
            MeetingRow mr = new MeetingRow();
            mr.EndingDate = this.EndingDate;
            if (this.ID >= 0)
                mr.SetID(this.ID);
            mr.MeetingData = this.MeetingData;
            mr.StartingDate = this.StartingDate;
            mr.Title = this.Title;
            return mr;
        }
        #endregion
    }
}
